@@ -48,7 +48,7 @@ class Node(_common.ListMixin, resource.Resource):
     _query_mapping = resource.QueryParameters(
         'associated', 'conductor_group', 'driver', 'fault',
         'provision_state', 'resource_class',
-        fields={'name': 'fields', 'type': _common.comma_separated_list},
+        fields={'type': _common.fields_type},
         instance_id='instance_uuid',
         is_maintenance='maintenance',
     )
@@ -487,7 +487,8 @@ class Node(_common.ListMixin, resource.Resource):
         elif not abort_on_failed_state:
             return False
 
-        if self.provision_state.endswith(' failed'):
+        if (self.provision_state.endswith(' failed') or
+                self.provision_state == 'error'):
             raise exceptions.ResourceFailure(
                 "Node %(node)s reached failure state \"%(state)s\"; "
                 "the last error is %(error)s" %
@@ -720,5 +721,114 @@ class Node(_common.ListMixin, resource.Resource):
                .format(node=self.id))
         exceptions.raise_from_response(response, error_message=msg)
 
+    def set_boot_device(self, session, boot_device, persistent=False):
+        """Set node boot device
+
+        :param session: The session to use for making this request.
+        :param boot_device: Boot device to assign to the node.
+        :param persistent: If the boot device change is maintained after node
+            reboot
+        :return: The updated :class:`~openstack.baremetal.v1.node.Node`
+        """
+        session = self._get_session(session)
+        version = self._get_microversion_for(session, 'commit')
+        request = self._prepare_request(requires_id=True)
+        request.url = utils.urljoin(request.url, 'management', 'boot_device')
+
+        body = {'boot_device': boot_device, 'persistent': persistent}
+
+        response = session.put(
+            request.url, json=body,
+            headers=request.headers, microversion=version,
+            retriable_status_codes=_common.RETRIABLE_STATUS_CODES)
+
+        msg = ("Failed to set boot device for node {node}"
+               .format(node=self.id))
+        exceptions.raise_from_response(response, error_message=msg)
+
+    def add_trait(self, session, trait):
+        """Add a trait to a node.
+
+        :param session: The session to use for making this request.
+        :param trait: The trait to add to the node.
+        :returns: The updated :class:`~openstack.baremetal.v1.node.Node`
+        """
+        session = self._get_session(session)
+        version = utils.pick_microversion(session, '1.37')
+        request = self._prepare_request(requires_id=True)
+        request.url = utils.urljoin(request.url, 'traits', trait)
+        response = session.put(
+            request.url, json=None,
+            headers=request.headers, microversion=version,
+            retriable_status_codes=_common.RETRIABLE_STATUS_CODES)
+
+        msg = ("Failed to add trait {trait} for node {node}"
+               .format(trait=trait, node=self.id))
+        exceptions.raise_from_response(response, error_message=msg)
+
+        self.traits = list(set(self.traits or ()) | {trait})
+
+    def remove_trait(self, session, trait, ignore_missing=True):
+        """Remove a trait from a node.
+
+        :param session: The session to use for making this request.
+        :param trait: The trait to remove from the node.
+        :param bool ignore_missing: When set to ``False``
+            :class:`~openstack.exceptions.ResourceNotFound` will be
+            raised when the trait does not exist.
+            Otherwise, ``False`` is returned.
+        :returns: The updated :class:`~openstack.baremetal.v1.node.Node`
+        """
+        session = self._get_session(session)
+        version = utils.pick_microversion(session, '1.37')
+        request = self._prepare_request(requires_id=True)
+        request.url = utils.urljoin(request.url, 'traits', trait)
+
+        response = session.delete(
+            request.url, headers=request.headers, microversion=version,
+            retriable_status_codes=_common.RETRIABLE_STATUS_CODES)
+
+        if ignore_missing or response.status_code == 400:
+            session.log.debug(
+                'Trait %(trait)s was already removed from node %(node)s',
+                {'trait': trait, 'node': self.id})
+            return False
+
+        msg = ("Failed to remove trait {trait} from bare metal node {node}"
+               .format(node=self.id, trait=trait))
+        exceptions.raise_from_response(response, error_message=msg)
+
+        self.traits = list(set(self.traits) - {trait})
+
+        return True
+
+    def set_traits(self, session, traits):
+        """Set traits for a node.
+
+        Removes any existing traits and adds the traits passed in to this
+        method.
+
+        :param session: The session to use for making this request.
+        :param traits: list of traits to add to the node.
+        :returns: The updated :class:`~openstack.baremetal.v1.node.Node`
+        """
+        session = self._get_session(session)
+        version = utils.pick_microversion(session, '1.37')
+        request = self._prepare_request(requires_id=True)
+        request.url = utils.urljoin(request.url, 'traits')
+
+        body = {'traits': traits}
+
+        response = session.put(
+            request.url, json=body,
+            headers=request.headers, microversion=version,
+            retriable_status_codes=_common.RETRIABLE_STATUS_CODES)
+
+        msg = ("Failed to set traits for node {node}"
+               .format(node=self.id))
+        exceptions.raise_from_response(response, error_message=msg)
+
+        self.traits = traits
+
 
 NodeDetail = Node
