@@ -183,6 +183,18 @@ class TestNodeWaitForProvisionState(base.TestCase):
                                self.node.wait_for_provision_state,
                                self.session, 'manageable')
 
+    def test_failure_error(self, mock_fetch):
+        def _get_side_effect(_self, session):
+            self.node.provision_state = 'error'
+            self.assertIs(session, self.session)
+
+        mock_fetch.side_effect = _get_side_effect
+
+        self.assertRaisesRegex(exceptions.ResourceFailure,
+                               'failure state "error"',
+                               self.node.wait_for_provision_state,
+                               self.session, 'manageable')
+
     def test_enroll_as_failure(self, mock_fetch):
         def _get_side_effect(_self, session):
             self.node.provision_state = 'enroll'
@@ -683,3 +695,67 @@ class TestNodeMaintenance(base.TestCase):
             json=None,
             headers=mock.ANY,
             microversion=mock.ANY)
+
+
+@mock.patch.object(node.Node, 'fetch', lambda self, session: self)
+@mock.patch.object(exceptions, 'raise_from_response', mock.Mock())
+class TestNodeSetBootDevice(base.TestCase):
+
+    def setUp(self):
+        super(TestNodeSetBootDevice, self).setUp()
+        self.node = node.Node(**FAKE)
+        self.session = mock.Mock(spec=adapter.Adapter,
+                                 default_microversion='1.1')
+
+    def test_node_set_boot_device(self):
+        self.node.set_boot_device(self.session, 'pxe', persistent=False)
+        self.session.put.assert_called_once_with(
+            'nodes/%s/management/boot_device' % self.node.id,
+            json={'boot_device': 'pxe', 'persistent': False},
+            headers=mock.ANY, microversion=mock.ANY,
+            retriable_status_codes=_common.RETRIABLE_STATUS_CODES)
+
+
+@mock.patch.object(node.Node, 'fetch', lambda self, session: self)
+@mock.patch.object(exceptions, 'raise_from_response', mock.Mock())
+class TestNodeTraits(base.TestCase):
+
+    def setUp(self):
+        super(TestNodeTraits, self).setUp()
+        self.node = node.Node(**FAKE)
+        self.session = mock.Mock(spec=adapter.Adapter,
+                                 default_microversion='1.37')
+        self.session.log = mock.Mock()
+
+    def test_node_add_trait(self):
+        self.node.add_trait(self.session, 'CUSTOM_FAKE')
+        self.session.put.assert_called_once_with(
+            'nodes/%s/traits/%s' % (self.node.id, 'CUSTOM_FAKE'),
+            json=None,
+            headers=mock.ANY, microversion='1.37',
+            retriable_status_codes=_common.RETRIABLE_STATUS_CODES)
+
+    def test_remove_trait(self):
+        self.node.remove_trait(self.session, 'CUSTOM_FAKE')
+        self.session.delete.assert_called_once_with(
+            'nodes/%s/traits/%s' % (self.node.id, 'CUSTOM_FAKE'),
+            headers=mock.ANY, microversion='1.37',
+            retriable_status_codes=_common.RETRIABLE_STATUS_CODES)
+
+    def test_remove_trait_missing(self):
+        self.session.delete.return_value.status_code = 400
+        self.assertFalse(self.node.remove_trait(self.session,
+                                                'CUSTOM_MISSING'))
+        self.session.delete.assert_called_once_with(
+            'nodes/%s/traits/%s' % (self.node.id, 'CUSTOM_MISSING'),
+            headers=mock.ANY, microversion='1.37',
+            retriable_status_codes=_common.RETRIABLE_STATUS_CODES)
+
+    def test_set_traits(self):
+        traits = ['CUSTOM_FAKE', 'CUSTOM_REAL', 'CUSTOM_MISSING']
+        self.node.set_traits(self.session, traits)
+        self.session.put.assert_called_once_with(
+            'nodes/%s/traits' % self.node.id,
+            json={'traits': ['CUSTOM_FAKE', 'CUSTOM_REAL', 'CUSTOM_MISSING']},
+            headers=mock.ANY, microversion='1.37',
+            retriable_status_codes=_common.RETRIABLE_STATUS_CODES)
